#!usr/bin/env python
"""message utils

make colorful text
print error message

Fore # BLACK, RED, GREEN, YELLOW, BLUE, MAGENTA, CYAN, WHITE, RESET.
Back # BLACK, RED, GREEN, YELLOW, BLUE, MAGENTA, CYAN, WHITE, RESET.
Style # DIM, NORMAL, BRIGHT, RESET_ALL
"""

from colorama import Fore


def colorful_text(text, color=Fore.RESET):
    """make target text colorful

    :param text: target text
    :param color
    :return: colored text
    """
    return color + text + Fore.RESET


def error_message(message='Ops, there are some error...'):
    """print the error message in red color

    :param message: error message
    :return: None
    """
    print(colorful_text(message, Fore.RED))



import random
from goods.goods_spider.spider import GoodsListSpider

GOODS = ('冰与火之歌', '权力的游戏', '移动硬盘-东芝', '海贼王', 'python', 'psv', 'ps4-港版', 'iphone6-32G-港版')


def convert_keywords(keywords):
    goods_keywords = keywords.split('-')
    assert isinstance(goods_keywords, list)
    key_words = '+'.join(goods_keywords)
    return key_words


def test_get_goods():
    random_index = random.randrange(0, len(GOODS))
    assert random_index < len(GOODS)
    goods = GOODS[random_index]
    assert goods in GOODS
    return convert_keywords(goods)


# def test_taobao():
#     goods = test_get_goods()
#     print(goods)
#     goods_list_spider = GoodsListSpider(goods)
#     result = goods_list_spider.fetch_goods()['taobao']()
#     assert result is not None
#     assert len(result) > 1
#
#
# def test_jd():
#     goods = test_get_goods()
#     print(goods)
#     goods_list_spider = GoodsListSpider(goods)
#     result = goods_list_spider.fetch_goods()['jd']()
#     assert result is not None
#     assert len(result) > 10



#!usr/bin/env python
import ssl
import re
import json
from datetime import date
from urllib import request, error, parse
from colorama import Fore
from bs4 import BeautifulSoup
from selenium import webdriver
from ..utils.message import colorful_text, error_message

ssl._create_default_https_context = ssl._create_unverified_context

JD_SEARCH = 'https://search.jd.com/Search?keyword={}&enc=utf-8&suggest=1.def.0&wq={}'
TAOBAO_SEARCH = 'https://s.taobao.com/search?q={keywords}&imgfile=&js=1' \
                '&stats_click=search_radio_all%3A1&initiative_id=staobaoz_{date}&ie=utf8'


class GoodsListSpider(object):
    def __init__(self, keywords):
        keywords = parse.quote(keywords)
        self.keywords = keywords
        self.headers = {
            'Connection': 'Keep-Alive',
            'Accept': 'text/html, application/xhtml+xml, */*',
            'Accept-Language': 'en-US,en;q=0.8,zh-Hans-CN;q=0.5,zh-Hans;q=0.3',
            'User-Agent': 'Mozilla/5.0 (X11; U; Linux i686) Gecko/20071127 Firefox/2.0.0.11',
        }
        self.url = None

    def fetch_goods(self):
        """web spider object creator

        :return: A object which contains different web spider
        """
        return {
            'jd': self.jd_spider,
            'taobao': self.taobao_spider
        }

    def jd_spider(self):
        self.url = JD_SEARCH.format(self.keywords, self.keywords)
        driver = webdriver.PhantomJS()
        driver.set_window_size(1024, 2000)
        driver.implicitly_wait(7)
        print(colorful_text('open jd search page....', color=Fore.RED))
        driver.get(self.url)
        return self.fetch_jd_goods(driver.page_source)

    @staticmethod
    def fetch_jd_goods(response):
        print(colorful_text('fetching goods from jd....', color=Fore.RED))
        search_result = None
        try:
            soup = BeautifulSoup(response, 'lxml')
            j_goods_list = soup.find('div', id="J_goodsList").find_all('li', attrs={"class": "gl-item"})

            if j_goods_list:
                search_result = []
                for j_goods_item in j_goods_list:
                    if j_goods_item.find('div', attrs={"class": "p-img"}):
                        j_goods = {
                            'url': j_goods_item.find('div', attrs={"class": "p-img"}).a.attrs["href"],
                            'intro': j_goods_item.find('div', attrs={"class": "p-name"}).a.attrs["title"],
                            'price': float(j_goods_item.find('div', attrs={"class": "p-price"})
                                           .find('strong').find('i').string.strip()),
                            'delivery': 'none',
                            'sales': int(j_goods_item.find('div', attrs={"class": "p-commit"})
                            .find('strong').find('a').string.strip()),
                            'belong': colorful_text('京东', Fore.CYAN)
                        }
                        search_result.append(j_goods)
            else:
                error_message('没有搜索到商品')
        except (error.HTTPError, error.URLError, error.ContentTooShortError):
            error_message()
        except UnicodeEncodeError:
            error_message('encoding error')
        finally:
            return search_result

    def taobao_spider(self):
        search_date = ''.join(str(date.today()).split('-'))
        self.url = TAOBAO_SEARCH.format(keywords=self.keywords, date=search_date)
        self.headers['GET'] = self.url
        fetch_taobao_result = self.fetch_taobao_goods()
        if fetch_taobao_result:
            return self.filter_taobao_data(fetch_taobao_result)
        else:
            return None

    def fetch_taobao_goods(self):
        """taobao spider

        :return: None or search result
        """
        search_result = None
        print(colorful_text('fetching goods from taobao....', color=Fore.MAGENTA))
        try:
            req = request.Request(self.url, headers=self.headers)
            response = request.urlopen(req).read().decode('UTF-8')
            re_result = re.findall(r'g_page_config\s*=\s*(.*);', str(response))[0]
            json_result = json.loads(re_result)

            # with open('./goods/goods_spider/taobao.html', encoding='utf-8') as f:
            #     response = f.read()
            # re_result = re.findall(r'g_page_config\s*=\s*(.*);', str(response))[0]
            # json_result = json.loads(re_result)

            if json_result['mods']['itemlist']['data']['auctions']:
                search_result = json_result['mods']['itemlist']['data']['auctions']
            else:
                error_message('没有搜索到商品')
        except (error.HTTPError, error.URLError, error.ContentTooShortError):
            error_message()
        except UnicodeEncodeError:
            error_message('encoding error')
        finally:
            return search_result

    @staticmethod
    def filter_taobao_data(search_result):
        """get target data in search result

        :param search_result
        :return: validate data list
        """

        return [{
                'intro': result["raw_title"],
                'price': float(result["view_price"]),
                'delivery': colorful_text(result["view_fee"], Fore.RED)
                if float(result["view_fee"]) > 0 else result["view_fee"],
                'sales': int(result["view_sales"].split('人')[0]),
                'belong': colorful_text("天猫", Fore.CYAN)
                if result["shopcard"] and result["shopcard"].get('isTmall', False) else "淘宝",
                'url': result["detail_url"]
                } for result in search_result]



#!usr/bin/env python
"""
Usage:
    $ goods [--site= all | jd | tb]

Options:
    --site: 要搜索的网站，jd-京东，tb-淘宝，默认全部

Example:
    $ goods --site=jd
    $ goods
"""
import click
from goods.entry import entry, WEB_NAME


@click.command()
@click.option('--goods', prompt='what do you wanna search')
@click.option('--site', type=click.Choice(['all', 'jd', 'tb']), default='all')
def get_train_args(goods, site):
    """get input from comment line

    :param goods
    :param site
    :return: None
    """
    goods_keywords = goods.split(' ')
    webs = WEB_NAME.get(site, WEB_NAME['all'])
    entry.get_goods(goods_keywords, webs)
    exit(0)


if __name__ == '__main__':
    get_train_args()



#!usr/bin/env python

import threading
from ..goods_spider.spider import GoodsListSpider
from ..utils.const_value import WEB_NAME


class GoodsThread(threading.Thread):
    def __init__(self, lock, all_goods, keywords, web):
        threading.Thread.__init__(self)
        self.lock = lock
        self.keywords = keywords
        self.web = web
        self.all_goods = all_goods

    def run(self):
        goods_list_spider = GoodsListSpider(self.keywords)
        if self.web in WEB_NAME.values():
            result = goods_list_spider.fetch_goods()[self.web]()
            if result:
                self.append_data(result)

    def append_data(self, data):
        self.lock.acquire()
        self.all_goods.extend(data)
        self.lock.release()



#!usr/bin/env python
"""
get data and print table package
"""
import webbrowser
from operator import itemgetter
from colorama import Fore
from prettytable import PrettyTable
from ..utils.message import colorful_text, error_message
from ..goods_threads import thread_pool


TABLE_TITLE = ('编号', '简介', '价格', '邮费', '购买人数', '所属')
ITEM_KEY = ('index', 'intro', 'price', 'delivery', 'sales', 'belong')


def get_goods(goods_keywords, webs):
    """get keywords and search

    :param goods_keywords: input keywords
    :param webs: webs which will be search
    :return: None
    """
    try:
        assert isinstance(goods_keywords, list)
    except AssertionError:
        error_message('expect to receive a list')
    key_words = '+'.join(goods_keywords)
    thread_pool.build_thread(key_words, webs=webs)
    print_goods(thread_pool.export_date())


def print_goods(search_result):
    """use validate search result to print a table

    :param search_result: search result in taobao and jd
    :return: None
    """
    search_result = sort_by_money(search_result)

    goods_table = PrettyTable(TABLE_TITLE)
    for index, goods in enumerate(search_result):
        goods["index"] = index
        goods_row = [goods[item] for item in ITEM_KEY]
        goods_table.add_row(goods_row)
    print(colorful_text('ready to hands chopping?', Fore.CYAN))
    print(goods_table)
    open_detail_page(search_result)


def sort_by_money(search_result):
    result = sorted(search_result, key=itemgetter('sales', 'price'), reverse=True)
    return result


def open_detail_page(filtered_goods):
    """expect a number or a string which joined by ','
    to open the target goods url in a browser window

    :param filtered_goods
    :return: None
    """
    print(colorful_text('which do you prefer? type it\'s index', Fore.MAGENTA))
    print(colorful_text('if many, use \',\' to split them', Fore.MAGENTA))
    print(colorful_text('use \'control + c\' to exit.', Fore.RED))
    try:
        index = input('goods index: ')
        result_goods = filter(get_target_goods(index.split(',')), filtered_goods)
        goods_list = [goods for goods in result_goods]

        if len(goods_list):
            for goods in goods_list:
                goods_url = goods["url"]
                if goods_url[0] == '/':
                    goods_url = 'https:{}'.format(goods_url)
                webbrowser.open_new(goods_url)
        else:
            error_message('no such index')
            open_detail_page(filtered_goods)
    except KeyboardInterrupt:
        error_message('exit')


def get_target_goods(indexs):
    """search in filtered_goods by user's input indexs

    :param indexs: user's input indexs
    :return: if is validate
    """
    def filter_goods(goods):
        return str(goods["index"]) in indexs
    return filter_goods



import threading
from .goods_thread import GoodsThread

threads = []
lock = threading.Lock()
all_goods = []


def build_thread(keywords, webs):
    """build thread by target web & put them into a thread pool

    :param keywords: search target
    :param webs: target shopping site
    :return:
    """
    global all_goods
    for web in webs:
        goods_thread = GoodsThread(lock, all_goods, keywords, web)
        threads.append(goods_thread)
        goods_thread.start()


def export_date():
    """finish all thread & get result data

    :return: search result
    """
    for thread in threads:
        if thread:
            thread.join()
    return all_goods



from setuptools import setup, find_packages
from goods import __version__

setup(
    name='hands_chopping',
    version=__version__,
    keywords=('taobao', 'jingdong', 'hands chopping', 'spider'),
    description='Search goods in shopping web, It\' time to chop your hands.',
    author='ecmadao',
    author_email='wlec@outlook.com',
    url='https://github.com/ecmadao/Hands-Chopping',
    packages=find_packages(),
    py_modules=['run'],
    include_package_data=True,
    platforms='any',
    install_requires=[
        'prettytable',
        'bs4',
        'selenium',
        'Click'
    ],
    entry_points={
        'console_scripts': ['goods=goods.__main__:main']
    },
    license='MIT',
    zip_safe=False,
    classifiers=[
         'Environment :: Console',
         'Programming Language :: Python',
         'Programming Language :: Python :: 3.5',
         'Programming Language :: Python :: Implementation :: CPython'
    ]
)
